//
//  FanBookApiObject.h
//  MSLDFanBookPackage
//
//  Created by berton.luo on 2021/3/1.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM (int, FanBookErrorCode) {
    FanBookSuccess                 =  0,   /** 成功    */
    FanBookErrorCodeCommon         = -1,  /** 普通错误类型    */
    FanBookErrorCodeUserCancel     = -2,  /** 用户点击取消并返回    */
    FanBookErrorCodeSentFail       = -3,  /** 发送失败    */
    FanBookErrorCodeAuthDeny       = -4,  /**  授权失败    */
    FanBookErrorCodeUnsupport      = -5,  /**  FanBook不支持    */
    FanBookErrorCodeUnknown        = -10000, /**  未知错误    */
    FanBookErrorCodeUninstalled    = -10001, /**  FanBook未安装    */
    FanBookErrorCodeParameterError = -10003, /**  参数缺失或参数错误    */
    FanBookErrorCodeInitError      = -10004, /**  尚未初始化或初始化失败    */
};

/**
 调用成功回调block，注意

 @param msg 提示信息
 @param data 方法返回值
 */
typedef void (^FanBookSDKSuccess)(NSString *msg, __nullable id data);

/**
 调用失败回调block

 @param code 错误码 FanBookErrorCode
 @param msg 提示信息
 @param errorData 错误对象
 */
typedef void (^FanBookSDKFail)(FanBookErrorCode code, NSString *msg, __nullable id errorData);

#pragma mark - FanBookApiDelegate

/// 接收并处理来自FanBook终端程序的事件消息
/// FanBookApiDelegate 会在handleOpenURL:delegate:中使用并触发

@class FanBookBaseResp;
@protocol FanBookApiDelegate <NSObject>
@optional

/// 发送一个sendReq后，收到FanBook的回应
///
/// @attention 必须在handleOpenURL: delegate:方法种传入delegate才会触发
/// 收到一个来自FanBook的处理结果。调用一次handleOpenURL: delegate:后会收到onResp。
/// 可能收到的处理结果有FanBookSendAuthResp FanBookShareResp等。
/// @param resp resp具体的回应内容
- (void)onResp:(FanBookBaseResp *)resp;

@end

#pragma mark - FanBookBaseReq
/*! @brief 该类为FanBook终端SDK所有请求类的基类
 *
 */
@interface FanBookBaseReq : NSObject

/** 请求类型 */
@property (nonatomic, assign) int type;
/** 第三方程序本身用来标识其请求的唯一性，最后跳转回第三方程序时，由FanBook终端回传。
 * @note state字符串长度不能超过1K
 */
@property (nonatomic, copy) NSString *state;

/** 三方应用在fanbook后台申请的服务器Id */
@property (nonatomic, copy) NSString *guildId;

/** 服务器邀请码。当玩家不在官方服务器时，使用此邀请码邀请玩家加入 */
@property (nonatomic, copy) NSString *inviteCode;

@end

#pragma mark - FanBookBaseResp
/*! @brief 该类为FanBook终端SDK所有响应类的基类
 *
 */
@interface FanBookBaseResp : NSObject
/** 错误码 */
@property (nonatomic, assign) int errCode;
/** 错误提示字符串 */
@property (nonatomic, copy) NSString *errStr;
/** 响应类型 */
@property (nonatomic, assign) int type;

/** 第三方程序发送时用来标识其请求的唯一性的标志，由第三方程序调用sendReq时传入，由FanBook终端回传
 * @note state字符串长度不能超过1K
 */
@property (nonatomic, copy, nullable) NSString *state;

@end

#pragma mark - FanBookSendAuthReq
/*! @brief 第三方程序向FanBook终端请求认证的消息结构
 *
 * 第三方程序要向FanBook申请认证，并请求某些权限，需要调用FanBookApi的sendReq成员函数，
 * 向FanBook终端发送一个FanBookSendAuthReq消息结构。FanBook终端处理完后会向第三方程序发送一个处理结果。
 * @see FanBookSendAuthResp
 */
@interface FanBookSendAuthReq : FanBookBaseReq
/** 第三方程序要向FanBook申请认证，并请求某些权限，需要调用FanBookApi的sendReq成员函数，向FanBook终端发送一个FanBookSendAuthReq消息结构。FanBook终端处理完后会向第三方程序发送一个处理结果。
 * @see FanBookSendAuthResp
 * @note scope字符串长度不能超过1K
 */
@property (nonatomic, copy) NSString *scope;

/** 由用户FanBook和AppID组成的唯一标识，需要校验FanBook用户是否换号登录时填写*/
@property (nonatomic, copy) NSString *openID;

@end

#pragma mark - FanBookSendAuthResp
/*! @brief FanBook处理完第三方程序的认证和权限申请后向第三方程序回送的处理结果。
 *
 * 第三方程序要向FanBook申请认证，并请求某些权限
 * FanBook终端处理完后会向第三方程序发送一个FanBookSendAuthResp。
 * @see onResp
 */
@interface FanBookSendAuthResp : FanBookBaseResp

@property (nonatomic, copy, nullable) NSString *lang;
@property (nonatomic, copy, nullable) NSString *country;
//授权码
@property (nonatomic, copy, nullable) NSString *code;
@end

@interface FanBookApiObject : NSObject

@end

#pragma mark - FanbookShareBaseReq
/*! @brief 该类为FanBook终端SDK所有分享请求类的基类
 *
 * fanbook分享
 * @note imageData成员不能为空
 * @see FanbookShareBaseReq
 */
@interface FanbookShareBaseReq : FanBookBaseReq

/** 分享的描述
 * @note 长度不能超过2K
 */
@property (nonatomic, copy, nullable) NSString *desc;

@end

#pragma mark - FanBookShareResp
/*! @brief FanBook处理完第三方程序的分享后向第三方程序回送的处理结果。
 *
 * 第三方程序要向FanBook分享图片，链接
 * FanBook终端处理完后会向第三方程序发送一个FanBookShareResp。
 * @see onResp
 */
@interface FanBookShareResp : FanBookBaseResp

@end

#pragma mark - FanbookShareImageReq
/*! @brief 第三方程序要向FanBook分享图片，需要调用FanBookApi的sendReq成员函数，向FanBook终端发送一个FanbookShareImageReq消息结构。FanBook终端处理完后会向第三方程序发送一个处理结果。
 *
 * fanbook分享内容中包含的图片数据对象。
 * @note imageData成员不能为空
 * @see FanbookShareImageReq
 */
@interface FanbookShareImageReq : FanbookShareBaseReq

/** 要分享的图片
 * @note 支持图片链接NSString：长度不能超过2KB /  图片的二进制数据NSData: 内容大小不超过10MB
 */
@property (nonatomic, copy, nonnull) id image;

@end

#pragma mark - FanbookShareLinkReq
/*! @brief 第三方程序要向FanBook分享链接，需要调用FanBookApi的sendReq成员函数，向FanBook终端发送一个FanbookShareLinkReq消息结构。FanBook终端处理完后会向第三方程序发送一个处理结果。
 *
 * fanbook分享内容中包含的图片数据对象。
 * @note imageData成员不能为空
 * @see FanbookShareLinkReq
 */
@interface FanbookShareLinkReq : FanbookShareBaseReq

/** 点击后跳转的链接
 * @note 不能为空且长度不能超过10K
 */
@property (nonatomic, copy, nonnull) NSString *link;

/** 分享链接的预览图片
 * @note 支持图片链接NSString：长度不能超过2KB /  图片的二进制数据NSData: 内容大小不超过10MB
 */
@property (nonatomic, copy, nonnull) id image;

@end

NS_ASSUME_NONNULL_END
